@@ -27,10 +27,15 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.lang.reflect.Field;
-import java.util.Hashtable;
-import java.util.Vector;
-import java.util.Enumeration;
 import java.net.URL;
+import java.util.ArrayList;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Hashtable;
+import java.util.List;
+import java.util.Map;
+import java.util.Vector;
+
 import javax.xml.parsers.ParserConfigurationException;
 import javax.xml.parsers.SAXParserFactory;
 import javax.xml.transform.ErrorListener;
@@ -38,26 +43,28 @@ import javax.xml.transform.Source;
 import javax.xml.transform.SourceLocator;
 import javax.xml.transform.Templates;
 import javax.xml.transform.Transformer;
+import javax.xml.transform.TransformerConfigurationException;
 import javax.xml.transform.TransformerException;
 import javax.xml.transform.TransformerFactory;
 import javax.xml.transform.URIResolver;
 import javax.xml.transform.sax.SAXSource;
 import javax.xml.transform.stream.StreamResult;
 import javax.xml.transform.stream.StreamSource;
-import javax.xml.transform.TransformerConfigurationException;
+
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.Project;
-import org.apache.tools.ant.taskdefs.XSLTLiaison3;
+import org.apache.tools.ant.taskdefs.XSLTLiaison4;
 import org.apache.tools.ant.taskdefs.XSLTLogger;
 import org.apache.tools.ant.taskdefs.XSLTLoggerAware;
 import org.apache.tools.ant.taskdefs.XSLTProcess;
-import org.apache.tools.ant.types.XMLCatalog;
 import org.apache.tools.ant.types.Resource;
+import org.apache.tools.ant.types.XMLCatalog;
 import org.apache.tools.ant.types.resources.FileProvider;
 import org.apache.tools.ant.types.resources.FileResource;
 import org.apache.tools.ant.types.resources.URLProvider;
 import org.apache.tools.ant.util.FileUtils;
 import org.apache.tools.ant.util.JAXPUtils;
+import org.apache.tools.ant.util.JavaEnvUtils;
 import org.xml.sax.EntityResolver;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
@@ -68,7 +75,7 @@ import org.xml.sax.XMLReader;
  *
  * @since Ant 1.3
  */
-public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware {
+public class TraXLiaison implements XSLTLiaison4, ErrorListener, XSLTLoggerAware {
 
     /**
      * Helper for transforming filenames to URIs.
@@ -115,13 +122,16 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
     private URIResolver uriResolver;
 
     /** transformer output properties */
-    private Vector outputProperties = new Vector();
+    private final Vector outputProperties = new Vector();
 
     /** stylesheet parameters */
-    private Hashtable params = new Hashtable();
+    private final Hashtable<String, Object> params = new Hashtable<String, Object>();
 
     /** factory attributes */
-    private Vector attributes = new Vector();
+    private final List<Object[]> attributes = new ArrayList<Object[]>();
+
+    /** factory features */
+    private final Map<String, Boolean> features = new HashMap<String, Boolean>();
 
     /** whether to suppress warnings */
     private boolean suppressWarnings = false;
@@ -133,7 +143,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * Constructor for TraXLiaison.
      * @throws Exception never
      */
-    public TraXLiaison() throws Exception {
+    public TraXLiaison() throws Exception { //NOSONAR
     }
 
     /**
@@ -141,8 +151,8 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * @param stylesheet a <code>File</code> value
      * @throws Exception on error
      */
-    public void setStylesheet(File stylesheet) throws Exception {
-        FileResource fr = new FileResource();
+    public void setStylesheet(final File stylesheet) throws Exception {
+        final FileResource fr = new FileResource();
         fr.setProject(project);
         fr.setFile(stylesheet);
         setStylesheet(fr);
@@ -153,7 +163,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * @param stylesheet a {@link org.apache.tools.ant.types.Resource} value
      * @throws Exception on error
      */
-    public void setStylesheet(Resource stylesheet) throws Exception {
+    public void setStylesheet(final Resource stylesheet) throws Exception {
         if (this.stylesheet != null) {
             // resetting the stylesheet - reset transformer
             transformer = null;
@@ -173,7 +183,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * @param outfile the result file
      * @throws Exception on error
      */
-    public void transform(File infile, File outfile) throws Exception {
+    public void transform(final File infile, final File outfile) throws Exception {
         if (transformer == null) {
             createTransformer();
         }
@@ -183,10 +193,10 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
         try {
             fis = new BufferedInputStream(new FileInputStream(infile));
             fos = new BufferedOutputStream(new FileOutputStream(outfile));
-            StreamResult res = new StreamResult(fos);
+            final StreamResult res = new StreamResult(fos);
             // not sure what could be the need of this...
             res.setSystemId(JAXPUtils.getSystemId(outfile));
-            Source src = getSource(fis, infile);
+            final Source src = getSource(fis, infile);
 
             // set parameters on each transformation, maybe something has changed
             //(e.g. value of file name parameter)
@@ -211,7 +221,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * satisfies the requested configuration.
      * @throws SAXException in case of problem detected by the SAX parser.
      */
-    private Source getSource(InputStream is, File infile)
+    private Source getSource(final InputStream is, final File infile)
         throws ParserConfigurationException, SAXException {
         // todo: is this comment still relevant ??
         // FIXME: need to use a SAXSource as the source for the transform
@@ -219,9 +229,9 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
         Source src = null;
         if (entityResolver != null) {
             if (getFactory().getFeature(SAXSource.FEATURE)) {
-                SAXParserFactory spFactory = SAXParserFactory.newInstance();
+                final SAXParserFactory spFactory = SAXParserFactory.newInstance();
                 spFactory.setNamespaceAware(true);
-                XMLReader reader = spFactory.newSAXParser().getXMLReader();
+                final XMLReader reader = spFactory.newSAXParser().getXMLReader();
                 reader.setEntityResolver(entityResolver);
                 src = new SAXSource(reader, new InputSource(is));
             } else {
@@ -237,7 +247,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
         return src;
     }
 
-    private Source getSource(InputStream is, Resource resource)
+    private Source getSource(final InputStream is, final Resource resource)
         throws ParserConfigurationException, SAXException {
         // todo: is this comment still relevant ??
         // FIXME: need to use a SAXSource as the source for the transform
@@ -245,9 +255,9 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
         Source src = null;
         if (entityResolver != null) {
             if (getFactory().getFeature(SAXSource.FEATURE)) {
-                SAXParserFactory spFactory = SAXParserFactory.newInstance();
+                final SAXParserFactory spFactory = SAXParserFactory.newInstance();
                 spFactory.setNamespaceAware(true);
-                XMLReader reader = spFactory.newSAXParser().getXMLReader();
+                final XMLReader reader = spFactory.newSAXParser().getXMLReader();
                 reader.setEntityResolver(entityResolver);
                 src = new SAXSource(reader, new InputSource(is));
             } else {
@@ -266,14 +276,14 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
         return src;
     }
 
-    private String resourceToURI(Resource resource) {
-        FileProvider fp = resource.as(FileProvider.class);
+    private String resourceToURI(final Resource resource) {
+        final FileProvider fp = resource.as(FileProvider.class);
         if (fp != null) {
             return FILE_UTILS.toURI(fp.getFile().getAbsolutePath());
         }
-        URLProvider up = resource.as(URLProvider.class);
+        final URLProvider up = resource.as(URLProvider.class);
         if (up != null) {
-            URL u = up.getURL();
+            final URL u = up.getURL();
             return String.valueOf(u);
         } else {
             return resource.getName();
@@ -296,7 +306,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
             xslStream
                 = new BufferedInputStream(stylesheet.getInputStream());
             templatesModTime = stylesheet.getLastModified();
-            Source src = getSource(xslStream, stylesheet);
+            final Source src = getSource(xslStream, stylesheet);
             templates = getFactory().newTemplates(src);
         } finally {
             if (xslStream != null) {
@@ -312,7 +322,8 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * @see #addParam(java.lang.String, java.lang.String)
      * @see #setOutputProperty(java.lang.String, java.lang.String)
      */
-    private void createTransformer() throws Exception {
+    private void createTransformer()
+        throws IOException, ParserConfigurationException, SAXException, TransformerException {
         if (templates == null) {
             readTemplates();
         }
@@ -331,19 +342,19 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
         }
 
         if (traceConfiguration != null) {
-            if ("org.apache.xalan.transformer.TransformerImpl"
+            if ("org.apache.xalan.transformer.TransformerImpl" //NOSONAR
                 .equals(transformer.getClass().getName())) {
                 try {
-                    Class traceSupport =
+                    final Class traceSupport =
                         Class.forName("org.apache.tools.ant.taskdefs.optional."
                                       + "Xalan2TraceSupport", true,
                                       Thread.currentThread()
                                       .getContextClassLoader());
-                    XSLTTraceSupport ts =
+                    final XSLTTraceSupport ts =
                         (XSLTTraceSupport) traceSupport.newInstance();
                     ts.configureTrace(transformer, traceConfiguration);
-                } catch (Exception e) {
-                    String msg = "Failed to enable tracing because of " + e;
+                } catch (final Exception e) {
+                    final String msg = "Failed to enable tracing because of " + e;
                     if (project != null) {
                         project.log(msg, Project.MSG_WARN);
                     } else {
@@ -351,7 +362,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
                     }
                 }
             } else {
-                String msg = "Not enabling trace support for transformer"
+                final String msg = "Not enabling trace support for transformer"
                     + " implementation" + transformer.getClass().getName();
                 if (project != null) {
                     project.log(msg, Project.MSG_WARN);
@@ -363,13 +374,13 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
     }
 
     /**
-     * Sets the paramters for the transformer.
+     * Sets the parameters for the transformer.
      */
     private void setTransformationParameters() {
         for (final Enumeration enumeration = params.keys();
              enumeration.hasMoreElements();) {
             final String name = (String) enumeration.nextElement();
-            final String value = (String) params.get(name);
+            final Object value = params.get(name);
             transformer.setParameter(name, value);
         }
     }
@@ -397,8 +408,8 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
                         Class.forName(factoryName, true,
                                       Thread.currentThread()
                                       .getContextClassLoader());
-                } catch (ClassNotFoundException cnfe) {
-                    String msg = "Failed to load " + factoryName
+                } catch (final ClassNotFoundException cnfe) {
+                    final String msg = "Failed to load " + factoryName
                         + " via the configured classpath, will try"
                         + " Ant's classpath instead.";
                     if (logger != null) {
@@ -414,44 +425,43 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
                     clazz = Class.forName(factoryName);
                 }
                 tfactory = (TransformerFactory) clazz.newInstance();
-            } catch (Exception e) {
+            } catch (final Exception e) {
                 throw new BuildException(e);
             }
         }
 
-        try { // #51668, #52382
-            Field _isNotSecureProcessing = tfactory.getClass().getDeclaredField("_isNotSecureProcessing");
-            _isNotSecureProcessing.setAccessible(true);
-            _isNotSecureProcessing.set(tfactory, Boolean.TRUE);
-        } catch (Exception x) {
-            if (project != null) {
-                project.log(x.toString(), Project.MSG_DEBUG);
-            }
-        }
+        applyReflectionHackForExtensionMethods();
 
         tfactory.setErrorListener(this);
 
         // specific attributes for the transformer
         final int size = attributes.size();
         for (int i = 0; i < size; i++) {
-            final Object[] pair = (Object[]) attributes.elementAt(i);
+            final Object[] pair = attributes.get(i);
             tfactory.setAttribute((String) pair[0], pair[1]);
         }
 
+        for (Map.Entry<String, Boolean> feature : features.entrySet()) {
+            try {
+                tfactory.setFeature(feature.getKey(), feature.getValue());
+            } catch (TransformerConfigurationException ex) {
+                throw new BuildException(ex);
+            }
+        }
+
         if (uriResolver != null) {
             tfactory.setURIResolver(uriResolver);
         }
         return tfactory;
     }
 
-
     /**
      * Set the factory name to use instead of JAXP default lookup.
      * @param name the fully qualified class name of the factory to use
      * or null for the default JAXP look up mechanism.
      * @since Ant 1.6
      */
-    public void setFactory(String name) {
+    public void setFactory(final String name) {
         factoryName = name;
     }
 
@@ -462,9 +472,19 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * string or object.
      * @since Ant 1.6
      */
-    public void setAttribute(String name, Object value) {
+    public void setAttribute(final String name, final Object value) {
         final Object[] pair = new Object[]{name, value};
-        attributes.addElement(pair);
+        attributes.add(pair);
+    }
+
+    /**
+     * Set a custom feature for the JAXP factory implementation.
+     * @param name the feature name.
+     * @param value the value of the feature
+     * @since Ant 1.9.8
+     */
+    public void setFeature(final String name, final boolean value) {
+        features.put(name, value);
     }
 
     /**
@@ -476,7 +496,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * @since Ant 1.5
      * @since Ant 1.5
      */
-    public void setOutputProperty(String name, String value) {
+    public void setOutputProperty(final String name, final String value) {
         final String[] pair = new String[]{name, value};
         outputProperties.addElement(pair);
     }
@@ -485,7 +505,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * Set the class to resolve entities during the transformation.
      * @param aResolver the resolver class.
      */
-    public void setEntityResolver(EntityResolver aResolver) {
+    public void setEntityResolver(final EntityResolver aResolver) {
         entityResolver = aResolver;
     }
 
@@ -493,7 +513,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * Set the class to resolve URIs during the transformation
      * @param aResolver a <code>EntityResolver</code> value
      */
-    public void setURIResolver(URIResolver aResolver) {
+    public void setURIResolver(final URIResolver aResolver) {
         uriResolver = aResolver;
     }
 
@@ -502,7 +522,17 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * @param name the name of the parameter
      * @param value the value of the parameter
      */
-    public void addParam(String name, String value) {
+    public void addParam(final String name, final String value) {
+        params.put(name, value);
+    }
+
+    /**
+     * Add a parameter.
+     * @param name the name of the parameter
+     * @param value the value of the parameter
+     * @since Ant 1.9.3
+     */
+    public void addParam(final String name, final Object value) {
         params.put(name, value);
     }
 
@@ -510,7 +540,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * Set a logger.
      * @param l a logger.
      */
-    public void setLogger(XSLTLogger l) {
+    public void setLogger(final XSLTLogger l) {
         logger = l;
     }
 
@@ -518,7 +548,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * Log an error.
      * @param e the exception to log.
      */
-    public void error(TransformerException e) {
+    public void error(final TransformerException e) {
         logError(e, "Error");
     }
 
@@ -526,7 +556,7 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * Log a fatal error.
      * @param e the exception to log.
      */
-    public void fatalError(TransformerException e) {
+    public void fatalError(final TransformerException e) {
         logError(e, "Fatal Error");
         throw new BuildException("Fatal error during transformation using " + stylesheet + ": " + e.getMessageAndLocation(), e);
     }
@@ -535,21 +565,21 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * Log a warning.
      * @param e the exception to log.
      */
-    public void warning(TransformerException e) {
+    public void warning(final TransformerException e) {
         if (!suppressWarnings) {
             logError(e, "Warning");
         }
     }
 
-    private void logError(TransformerException e, String type) {
+    private void logError(final TransformerException e, final String type) {
         if (logger == null) {
             return;
         }
 
-        StringBuffer msg = new StringBuffer();
-        SourceLocator locator = e.getLocator();
+        final StringBuffer msg = new StringBuffer();
+        final SourceLocator locator = e.getLocator();
         if (locator != null) {
-            String systemid = locator.getSystemId();
+            final String systemid = locator.getSystemId();
             if (systemid != null) {
                 String url = systemid;
                 if (url.startsWith("file:")) {
@@ -559,11 +589,11 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
             } else {
                 msg.append("Unknown file");
             }
-            int line = locator.getLineNumber();
+            final int line = locator.getLineNumber();
             if (line != -1) {
                 msg.append(":");
                 msg.append(line);
-                int column = locator.getColumnNumber();
+                final int column = locator.getColumnNumber();
                 if (column != -1) {
                     msg.append(":");
                     msg.append(column);
@@ -589,32 +619,37 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
      * @deprecated since 1.5.x.
      *             Use org.apache.tools.ant.util.JAXPUtils#getSystemId instead.
      */
-    protected String getSystemId(File file) {
+    @Deprecated
+    protected String getSystemId(final File file) {
         return JAXPUtils.getSystemId(file);
     }
 
 
     /**
      * Specific configuration for the TRaX liaison.
-     * @param xsltTask the XSLTProcess task instance from which this liasion
+     * @param xsltTask the XSLTProcess task instance from which this liaison
      *        is to be configured.
      */
-    public void configure(XSLTProcess xsltTask) {
+    public void configure(final XSLTProcess xsltTask) {
         project = xsltTask.getProject();
-        XSLTProcess.Factory factory = xsltTask.getFactory();
+        final XSLTProcess.Factory factory = xsltTask.getFactory();
         if (factory != null) {
             setFactory(factory.getName());
 
             // configure factory attributes
-            for (Enumeration attrs = factory.getAttributes();
+            for (final Enumeration attrs = factory.getAttributes();
                     attrs.hasMoreElements();) {
-                XSLTProcess.Factory.Attribute attr =
+                final XSLTProcess.Factory.Attribute attr =
                         (XSLTProcess.Factory.Attribute) attrs.nextElement();
                 setAttribute(attr.getName(), attr.getValue());
             }
+            for (final XSLTProcess.Factory.Feature feature
+                     : factory.getFeatures()) {
+                setFeature(feature.getName(), feature.getValue());
+            }
         }
 
-        XMLCatalog xmlCatalog = xsltTask.getXMLCatalog();
+        final XMLCatalog xmlCatalog = xsltTask.getXMLCatalog();
         // use XMLCatalog as the entity resolver and URI resolver
         if (xmlCatalog != null) {
             setEntityResolver(xmlCatalog);
@@ -623,9 +658,9 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
 
 
         // configure output properties
-        for (Enumeration props = xsltTask.getOutputProperties();
+        for (final Enumeration props = xsltTask.getOutputProperties();
                 props.hasMoreElements();) {
-            XSLTProcess.OutputProperty prop
+            final XSLTProcess.OutputProperty prop
                 = (XSLTProcess.OutputProperty) props.nextElement();
             setOutputProperty(prop.getName(), prop.getValue());
         }
@@ -634,4 +669,21 @@ public class TraXLiaison implements XSLTLiaison3, ErrorListener, XSLTLoggerAware
 
         traceConfiguration = xsltTask.getTraceConfiguration();
     }
+
+    private void applyReflectionHackForExtensionMethods() {
+        // Jigsaw doesn't allow reflection to work, so we can stop trying
+        if (JavaEnvUtils.isAtLeastJavaVersion(JavaEnvUtils.JAVA_1_7)
+            && !JavaEnvUtils.isAtLeastJavaVersion(JavaEnvUtils.JAVA_9)) {
+            try { // #51668, #52382
+                final Field _isNotSecureProcessing = tfactory.getClass().getDeclaredField("_isNotSecureProcessing");
+                _isNotSecureProcessing.setAccessible(true);
+                _isNotSecureProcessing.set(tfactory, Boolean.TRUE);
+            } catch (final Exception x) {
+                if (project != null) {
+                    project.log(x.toString(), Project.MSG_DEBUG);
+                }
+            }
+        }
+    }
+
 }
