@@ -29,6 +29,7 @@ import java.io.OutputStream;
 import java.util.Hashtable;
 import java.util.Iterator;
 import java.util.Vector;
+
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.taskdefs.ExecTask;
@@ -415,7 +416,7 @@ public class BorlandDeploymentTool extends GenericDeploymentTool
 
         if (java2iioparams != null) {
             log("additional  " + java2iioparams + " to java2iiop ", 0);
-            commandline.createArgument().setValue(java2iioparams);
+            commandline.createArgument().setLine(java2iioparams);
         }
 
 
@@ -484,6 +485,7 @@ public class BorlandDeploymentTool extends GenericDeploymentTool
         if (generateclient) {
             generateClient(jarFile);
         }
+        genfiles.clear();
     }
 
     /**
