@@ -40,10 +40,13 @@ public class Native2AsciiAdapterFactory {
      * vendor
      */
     public static String getDefault() {
-        if (JavaEnvUtils.isKaffe()) {
+        if (shouldUseKaffee()) {
             return KaffeNative2Ascii.IMPLEMENTATION_NAME;
         }
-        return SunNative2Ascii.IMPLEMENTATION_NAME;
+        if (shouldUseSun()) {
+            return SunNative2Ascii.IMPLEMENTATION_NAME;
+        }
+        return BuiltinNative2Ascii.IMPLEMENTATION_NAME;
     }
 
     /**
@@ -79,11 +82,14 @@ public class Native2AsciiAdapterFactory {
                                                  ProjectComponent log,
                                                  Path classpath)
         throws BuildException {
-        if ((JavaEnvUtils.isKaffe() && choice == null)
+        if ((shouldUseKaffee() && choice == null)
             || KaffeNative2Ascii.IMPLEMENTATION_NAME.equals(choice)) {
             return new KaffeNative2Ascii();
-        } else if (SunNative2Ascii.IMPLEMENTATION_NAME.equals(choice)) {
+        } else if ((shouldUseSun() && choice == null)
+                   || SunNative2Ascii.IMPLEMENTATION_NAME.equals(choice)) {
             return new SunNative2Ascii();
+        } else if (BuiltinNative2Ascii.IMPLEMENTATION_NAME.equals(choice)) {
+            return new BuiltinNative2Ascii();
         } else if (choice != null) {
             return resolveClassName(choice,
                                     // Memory leak in line below
@@ -91,9 +97,7 @@ public class Native2AsciiAdapterFactory {
                                     .createClassLoader(classpath));
         }
 
-        // This default has been good enough until Ant 1.6.3, so stick
-        // with it
-        return new SunNative2Ascii();
+        return new BuiltinNative2Ascii();
     }
 
     /**
@@ -113,4 +117,15 @@ public class Native2AsciiAdapterFactory {
             Native2AsciiAdapterFactory.class.getClassLoader(),
             Native2AsciiAdapter.class);
     }
+
+    private static final boolean shouldUseKaffee() {
+        return JavaEnvUtils.isKaffe() || JavaEnvUtils.isClasspathBased();
+    }
+
+    private static final boolean shouldUseSun() {
+        return JavaEnvUtils.isJavaVersion(JavaEnvUtils.JAVA_1_5)
+            || JavaEnvUtils.isJavaVersion(JavaEnvUtils.JAVA_1_6)
+            || JavaEnvUtils.isJavaVersion(JavaEnvUtils.JAVA_1_7)
+            || JavaEnvUtils.isJavaVersion(JavaEnvUtils.JAVA_1_8);
+    }
 }
