@@ -18,36 +18,37 @@
 package org.apache.tools.ant.types;
 
 import java.io.File;
+import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Enumeration;
-import java.util.Iterator;
+import java.util.List;
 import java.util.Stack;
-import java.util.Vector;
 
-import org.apache.tools.ant.Project;
-import org.apache.tools.ant.FileScanner;
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.DirectoryScanner;
-import org.apache.tools.ant.types.selectors.OrSelector;
+import org.apache.tools.ant.FileScanner;
+import org.apache.tools.ant.Project;
 import org.apache.tools.ant.types.selectors.AndSelector;
-import org.apache.tools.ant.types.selectors.NotSelector;
-import org.apache.tools.ant.types.selectors.DateSelector;
-import org.apache.tools.ant.types.selectors.FileSelector;
-import org.apache.tools.ant.types.selectors.NoneSelector;
-import org.apache.tools.ant.types.selectors.SizeSelector;
-import org.apache.tools.ant.types.selectors.TypeSelector;
-import org.apache.tools.ant.types.selectors.DepthSelector;
-import org.apache.tools.ant.types.selectors.DependSelector;
-import org.apache.tools.ant.types.selectors.ExtendSelector;
-import org.apache.tools.ant.types.selectors.SelectSelector;
-import org.apache.tools.ant.types.selectors.PresentSelector;
-import org.apache.tools.ant.types.selectors.SelectorScanner;
+import org.apache.tools.ant.types.selectors.ContainsRegexpSelector;
 import org.apache.tools.ant.types.selectors.ContainsSelector;
+import org.apache.tools.ant.types.selectors.DateSelector;
+import org.apache.tools.ant.types.selectors.DependSelector;
+import org.apache.tools.ant.types.selectors.DepthSelector;
+import org.apache.tools.ant.types.selectors.DifferentSelector;
+import org.apache.tools.ant.types.selectors.ExtendSelector;
+import org.apache.tools.ant.types.selectors.FileSelector;
 import org.apache.tools.ant.types.selectors.FilenameSelector;
 import org.apache.tools.ant.types.selectors.MajoritySelector;
-import org.apache.tools.ant.types.selectors.DifferentSelector;
-import org.apache.tools.ant.types.selectors.SelectorContainer;
-import org.apache.tools.ant.types.selectors.ContainsRegexpSelector;
+import org.apache.tools.ant.types.selectors.NoneSelector;
+import org.apache.tools.ant.types.selectors.NotSelector;
+import org.apache.tools.ant.types.selectors.OrSelector;
+import org.apache.tools.ant.types.selectors.PresentSelector;
 import org.apache.tools.ant.types.selectors.ReadableSelector;
+import org.apache.tools.ant.types.selectors.SelectSelector;
+import org.apache.tools.ant.types.selectors.SelectorContainer;
+import org.apache.tools.ant.types.selectors.SelectorScanner;
+import org.apache.tools.ant.types.selectors.SizeSelector;
+import org.apache.tools.ant.types.selectors.TypeSelector;
 import org.apache.tools.ant.types.selectors.WritableSelector;
 import org.apache.tools.ant.types.selectors.modifiedselector.ModifiedSelector;
 
@@ -62,10 +63,11 @@ public abstract class AbstractFileSet extends DataType
     implements Cloneable, SelectorContainer {
 
     private PatternSet defaultPatterns = new PatternSet();
-    private Vector additionalPatterns = new Vector();
-    private Vector selectors = new Vector();
+    private List<PatternSet> additionalPatterns = new ArrayList<PatternSet>();
+    private List<FileSelector> selectors = new ArrayList<FileSelector>();
 
     private File dir;
+    private boolean fileAttributeUsed;
     private boolean useDefaultExcludes = true;
     private boolean caseSensitive = true;
     private boolean followSymlinks = true;
@@ -130,6 +132,9 @@ public abstract class AbstractFileSet extends DataType
         if (isReference()) {
             throw tooManyAttributes();
         }
+        if (fileAttributeUsed && !getDir().equals(dir)) {
+            throw dirAndFileAreMutuallyExclusive();
+        }
         this.dir = dir;
         directoryScanner = null;
     }
@@ -165,7 +170,7 @@ public abstract class AbstractFileSet extends DataType
             throw noChildrenAllowed();
         }
         PatternSet patterns = new PatternSet();
-        additionalPatterns.addElement(patterns);
+        additionalPatterns.add(patterns);
         directoryScanner = null;
         return patterns;
     }
@@ -227,7 +232,11 @@ public abstract class AbstractFileSet extends DataType
         if (isReference()) {
             throw tooManyAttributes();
         }
+        if (getDir() != null) {
+            throw dirAndFileAreMutuallyExclusive();
+        }
         setDir(file.getParentFile());
+        fileAttributeUsed = true;
         createInclude().setName(file.getName());
     }
 
@@ -444,7 +453,7 @@ public abstract class AbstractFileSet extends DataType
      }
 
     /**
-     * Gets whether an error is/shold be thrown if the base directory
+     * Gets whether an error is/should be thrown if the base directory
      * does not exist.
      * @since Ant 1.8.2
      */
@@ -579,9 +588,7 @@ public abstract class AbstractFileSet extends DataType
         if (defaultPatterns.hasPatterns(getProject())) {
             return true;
         }
-        Enumeration e = additionalPatterns.elements();
-        while (e.hasMoreElements()) {
-            PatternSet ps = (PatternSet) e.nextElement();
+        for (PatternSet ps : additionalPatterns) {
             if (ps.hasPatterns(getProject())) {
                 return true;
             }
@@ -621,12 +628,12 @@ public abstract class AbstractFileSet extends DataType
      *
      * @return an <code>Enumeration</code> of selectors.
      */
-    public synchronized Enumeration selectorElements() {
+    public synchronized Enumeration<FileSelector> selectorElements() {
         if (isReference()) {
             return getRef(getProject()).selectorElements();
         }
         dieOnCircularReference();
-        return selectors.elements();
+        return Collections.enumeration(selectors);
     }
 
     /**
@@ -638,7 +645,7 @@ public abstract class AbstractFileSet extends DataType
         if (isReference()) {
             throw noChildrenAllowed();
         }
-        selectors.addElement(selector);
+        selectors.add(selector);
         directoryScanner = null;
         setChecked(false);
     }
@@ -799,7 +806,7 @@ public abstract class AbstractFileSet extends DataType
     }
 
     /**
-     * Add an arbitary selector.
+     * Add an arbitrary selector.
      * @param selector the <code>FileSelector</code> to add.
      * @since Ant 1.6
      */
@@ -844,13 +851,11 @@ public abstract class AbstractFileSet extends DataType
             try {
                 AbstractFileSet fs = (AbstractFileSet) super.clone();
                 fs.defaultPatterns = (PatternSet) defaultPatterns.clone();
-                fs.additionalPatterns = new Vector(additionalPatterns.size());
-                Enumeration e = additionalPatterns.elements();
-                while (e.hasMoreElements()) {
-                    fs.additionalPatterns
-                        .addElement(((PatternSet) e.nextElement()).clone());
+                fs.additionalPatterns = new ArrayList<PatternSet>(additionalPatterns.size());
+                for (PatternSet ps : additionalPatterns) {
+                    fs.additionalPatterns.add((PatternSet) ps.clone());
                 }
-                fs.selectors = new Vector(selectors);
+                fs.selectors = new ArrayList<FileSelector>(selectors);
                 return fs;
             } catch (CloneNotSupportedException e) {
                 throw new BuildException(e);
@@ -898,13 +903,12 @@ public abstract class AbstractFileSet extends DataType
         PatternSet ps = (PatternSet) defaultPatterns.clone();
         final int count = additionalPatterns.size();
         for (int i = 0; i < count; i++) {
-            Object o = additionalPatterns.elementAt(i);
-            ps.append((PatternSet) o, p);
+            ps.append(additionalPatterns.get(i), p);
         }
         return ps;
     }
 
-    protected synchronized void dieOnCircularReference(Stack stk, Project p)
+    protected synchronized void dieOnCircularReference(Stack<Object> stk, Project p)
         throws BuildException {
         if (isChecked()) {
             return;
@@ -912,17 +916,19 @@ public abstract class AbstractFileSet extends DataType
         if (isReference()) {
             super.dieOnCircularReference(stk, p);
         } else {
-            for (Iterator i = selectors.iterator(); i.hasNext(); ) {
-                Object o = i.next();
-                if (o instanceof DataType) {
-                    pushAndInvokeCircularReferenceCheck((DataType) o, stk, p);
+            for (FileSelector fileSelector : selectors) {
+                if (fileSelector instanceof DataType) {
+                    pushAndInvokeCircularReferenceCheck((DataType) fileSelector, stk, p);
                 }
             }
-            for (Iterator i = additionalPatterns.iterator(); i.hasNext(); ) {
-                PatternSet ps = (PatternSet) i.next();
+            for (PatternSet ps : additionalPatterns) {
                 pushAndInvokeCircularReferenceCheck(ps, stk, p);
             }
             setChecked(true);
         }
     }
+
+    private BuildException dirAndFileAreMutuallyExclusive() {
+        return new BuildException("you can only specify one of the dir and file attributes");
+    }
 }
