@@ -361,7 +361,7 @@ public class ZipEntry extends java.util.zip.ZipEntry implements Cloneable {
     private ZipExtraField[] getMergedFields() {
         final ZipExtraField[] zipExtraFields =
             copyOf(extraFields, extraFields.length + 1);
-        zipExtraFields[zipExtraFields.length] = unparseableExtra;
+        zipExtraFields[extraFields.length] = unparseableExtra;
         return zipExtraFields;
     }
 
@@ -517,7 +517,7 @@ public class ZipEntry extends java.util.zip.ZipEntry implements Cloneable {
             mergeExtraFields(local, true);
         } catch (final ZipException e) {
             // actually this is not be possible as of Ant 1.8.1
-            throw new RuntimeException("Error parsing extra fields for entry: "
+            throw new RuntimeException("Error parsing extra fields for entry: " //NOSONAR
                                        + getName() + " - " + e.getMessage(), e);
         }
     }
@@ -544,7 +544,7 @@ public class ZipEntry extends java.util.zip.ZipEntry implements Cloneable {
                                       ExtraFieldUtils.UnparseableExtraField.READ);
             mergeExtraFields(central, false);
         } catch (final ZipException e) {
-            throw new RuntimeException(e.getMessage(), e);
+            throw new RuntimeException(e.getMessage(), e); //NOSONAR
         }
     }
 
